/*
 * Copyright (C) 2008 KenD00
 * 
 * This file is part of ConvTableView.
 * 
 * ConvTableView is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package convtableview.gui;

import java.util.Enumeration;
import javax.swing.table.AbstractTableModel;
import javax.swing.tree.TreeNode;

import dumphd.bdplus.Segment;
import dumphd.util.Utils;


public class SegmentWrapper extends AbstractTableModel implements TreeNode {

   private final static String columnNames[] = { "#", "Base", "RDT", "RDA", "Adjust 0", "Adjust 1", "Offset 0", "Offset 1", "+Address 0", "+Address 1", "Patch 0", "Patch 1" };

   private TreeNode parent = null;
   private String nodeName = "";

   private Segment segment = null;


   public SegmentWrapper(Segment segment, TreeNode parent) {
      this.parent = parent;
      this.segment = segment;
      if (segment != null) {
         // Build the node name
         String namePrefix = null;
         if (segment.size() == 0) {
            namePrefix = "(empty) ";
         } else {
            namePrefix = "";
         }
         nodeName = String.format("0x%1$08X: %2$sSegment %3$d", segment.getSegmentOffset(), namePrefix, segment.getSegmentIndex());
         // Some informal checks
         long lastAddress = 0;
         for (int i = 0; i < segment.size(); i++) {
            // Check Repair Descriptor Type
            int rdt = (segment.getFlags(i) >> 6) & 0x3;
            if (rdt != 1 && rdt != 2) {
               Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Invalid Repair Descriptor Type 0x%4$1X", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i, rdt));
            }
            // Calculate difference between Patch0 and Patch1 AU
            long auDiff = segment.getAddress1(i) / 6144 - segment.getAddress0(i) / 6144;
            if (auDiff > 0) {
               // This happens often, suppress the message
               // Can't use parent.toString() to get the parent node name (== the TableID) because the name gets build after all segments are parsed 
               //Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Diff: %4$d - Patches are in different AUs", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i, auDiff));
            }
            // Check if current smallest address is bigger than the old largest address (== entries are ordered)
            if (segment.getAddress0(i) < lastAddress) {
               Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Patch address smaller than previous one", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i));
            }
            lastAddress = segment.getAddress1(i);
            // Check if Patch0 exceeds TS Packet Boundary
            if (187L - (segment.getAddress0(i) % 192) < 0) {
               Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Patch0 exceeds TS packet boundary", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i));
            }
            // Check if Patch1 exceeds TS Packet Boundary
            if (187L - (segment.getAddress1(i) % 192) < 0) {
               Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Patch1 exceeds TS packet boundary", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i));
            }
            // Check if Patch0 exceeds Aligned Unit Boundary
            if (6193L - (segment.getAddress0(i) % 6144) < 0) {
               Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Patch0 exceeds AU boundary", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i));
            }
            // Check if Patch1 exceeds Aligned Unit Boundary
            if (6139L - (segment.getAddress1(i) % 6144) < 0) {
               Utils.getMessagePrinter().println(String.format("TableID %1$05d Segment %2$d Entry %3$d: Patch1 exceeds AU boundary", ((SubTableWrapper)parent).getSubTable().getTableId(), segment.getSegmentIndex(), i));
            }
         }
      }
   }

   public String toString() {
      return nodeName;
   }

   public Segment getSegment() {
      return segment;
   }


   // ################################################################################################

   // ##########################
   // TreeNode Interface Methods
   // ##########################

   public Enumeration<TreeNode> children() {
      return new Enumeration<TreeNode>() {

         public boolean hasMoreElements() {
            return false;
         }

         public TreeNode nextElement() {
            return null;
         }

      };
   }

   public boolean getAllowsChildren() {
      return false;
   }

   public TreeNode getChildAt(int childIndex) {
      return null;
   }

   public int getChildCount() {
      return 0;
   }

   public int getIndex(TreeNode node) {
      return -1;
   }

   public TreeNode getParent() {
      return parent;
   }

   public boolean isLeaf() {
      return true;
   }


   // ################################################################################################

   // ############################
   // TableModel-Interface Methods
   // ############################

   public Class<?> getColumnClass(int columnIndex) {
      return String.class;
   }

   public String getColumnName(int columnIndex) {
      return columnNames[columnIndex];
   }

   /* (non-Javadoc)
    * @see javax.swing.table.TableModel#getColumnCount()
    */
   public int getColumnCount() {
      return columnNames.length;
   }

   /* (non-Javadoc)
    * @see javax.swing.table.TableModel#getRowCount()
    */
   public int getRowCount() {
      if (segment != null) {
         return segment.size();
      } else {
         return 0;
      }
   }

   /* (non-Javadoc)
    * @see javax.swing.table.TableModel#getValueAt(int, int)
    */
   public Object getValueAt(int rowIndex, int columnIndex) {
      switch (columnIndex) {
      case 0:
         return String.format("%1$d", rowIndex);
      case 1:
         return String.format("0x%1$08X", segment.getBase(rowIndex));
      case 2:
         return String.format("0x%1$1X", (segment.getFlags(rowIndex) >> 6) & 0x3);
      case 3:
         return String.format("0x%1$02X", segment.getFlags(rowIndex) & 0x3F);
      case 4:
         return String.format("0x%1$03X", segment.getAdjust0(rowIndex));
      case 5:
         return String.format("0x%1$03X", segment.getAdjust1(rowIndex));
      case 6:
         return String.format("0x%1$02X", segment.getOffset0(rowIndex));
      case 7:
         return String.format("0x%1$02X", segment.getOffset1(rowIndex));
      case 8:
         return String.format("0x%1$08X", segment.getAddress0(rowIndex));
      case 9:
         return String.format("0x%1$08X", segment.getAddress1(rowIndex));
      case 10:
         segment.getPatch0(rowIndex, Utils.buffer, 0);
         return String.format("0x%1$s", Utils.toHexString(Utils.buffer, 0, Segment.PATCH_LENGTH));
      case 11:
         segment.getPatch1(rowIndex, Utils.buffer, 0);
         return String.format("0x%1$s", Utils.toHexString(Utils.buffer, 0, Segment.PATCH_LENGTH));
      default:
         return "";
      }
   }

   public boolean isCellEditable(int rowIndex, int columnIndex) {
      return false;
   }

   // ################################################################################################

}